import { __redundancy_query__ } from "./private/redundancy";
import config from "../config/conf"

// 前端接口
const dp = {};

// 更新数据逻辑: 相对路径 `/${window.pageConfig.appType}/v1/form/searchFormDatas.json`
dp.updateForm = async function ({ formInstId, updateData, dpRemote = "updateForm", isLoading = true, hideToast, message = "操作成功", } = {}, { title, content } = {}) {
  if (!formInstId) {
    throw new Error(`The params of formInstId is empty.`);
  }
  // 确认弹出框
  if (title || content) await mjs.com.showConfirm(title, content);
  if (isLoading) mjs.com.showLoading();
  const rsp = await mjs.$this.dataSourceMap[dpRemote].load({ formInstId, updateFormDataJson: JSON.stringify(updateData) }).catch((error) => {
    if (!hideToast) mjs.com.showErrorMessage(error.message);
  });
  if (isLoading) mjs.com.hideLoading();
  if (!rsp) return;
  if (rsp && !hideToast) mjs.com.showMessage(message);
};

// 查询实例列表逻辑: 相对路径 `/${window.pageConfig.appType}/v1/form/searchFormDatas.json`
dp.queryForm = async function ({ formUuid, conditions = [], dpRemote = "queryForm", isLoading = true, hideToast = true, matchAllCondition = false, message = "查询成功" } = {}, { currentPage = 1, pageSize = config.pageSize, queryAll } = {}) {
  // 查询条件格式化
  const searchCondition = conditions.reduce((acc, cur) => {
    const value = __redundancy_query__(mjs.$this, cur);
    if (value) {
      acc[cur.src] = value;
    }
    return acc;
  }, {});
  const searchLength = Object.keys(searchCondition).length;
  if (conditions.length && !searchLength) return [];
  // 是否强制匹配多参必须都有值
  if (matchAllCondition && searchLength != conditions.length) return [];
  const queryFunc = async (currentPage) => {
    const params = {
      searchFieldJson: JSON.stringify(searchCondition),
      currentPage,
      pageSize,
    };
    // 兼容默认参数已设置FormUuid, 为空会被覆盖
    if (formUuid) params.formUuid = formUuid;
    return await mjs.$this.dataSourceMap[dpRemote].load(params).catch((error) => {
      if (!hideToast) mjs.com.showErrorMessage(error.message);
    });
  }
  if (isLoading) mjs.com.showLoading();
  const resp = await queryFunc(currentPage);
  const list = resp.data;
  const pages = Math.ceil(resp.totalCount / pageSize);
  if (queryAll && pages > currentPage) {
    const promiseReq = [];
    const pageCount = Math.ceil(config.upperLimit / pageSize);
    for (let index = currentPage + 1; index < pageCount + currentPage && index <= pages; index++) {
      promiseReq.push(queryFunc(index))
    }
    const respArr = await Promise.all(promiseReq)
    respArr.forEach(rsp => {
      list.push(...rsp.data)
    })
  }
  if (isLoading) mjs.com.hideLoading();
  if (resp && !hideToast) mjs.com.showMessage(message);
  return { totalCount: resp.totalCount, data: list }
};

// 明细全量查询: Promise并发
dp.queryDetail = async function ({ formUuid, conditions = {}, dpRemote = "queryForm" } = {}, { currentPage = 1, pageSize = config.pageSize, detailCount = config.detailCount } = {}) {
  const searchCondition = conditions.reduce((acc, cur) => {
    const value = __redundancy_query__(mjs.$this, cur);
    if (value) {
      acc[cur.src] = value;
    }
    return acc;
  }, {});
  const queryFunc = async (currentPage) => {
    const params = {
      searchFieldJson: JSON.stringify(searchCondition),
      currentPage,
      pageSize,
    };
    // 兼容默认参数已设置FormUuid, 为空会被覆盖
    if (formUuid) params.formUuid = formUuid;
    return await mjs.$this.dataSourceMap[dpRemote].load(params);
  }
  mjs.com.showLoading();
  const resp = await queryFunc(currentPage)
  const list = resp.data;
  const pages = Math.ceil(resp.totalCount / pageSize);
  if (pages > currentPage) {
    const promiseReq = [];
    const pageCount = Math.ceil(detailCount / pageSize);
    for (let index = currentPage + 1; index < pageCount + currentPage && index <= pages; index++) {
      promiseReq.push(queryFunc(index))
    }
    const respArr = await Promise.all(promiseReq)
    respArr.forEach(res => {
      list.push(...res.data)
    })
  }
  mjs.com.hideLoading();
  return { totalCount: resp.totalCount, data: list }
}

export default dp;
