// 组件数据处理

// 接口返回取值到组件, 人员搜索框需要对象类型
export function __redundancy_value__ (row, prop, def) {
  // 为空返回默认值
  if (!prop) return def;
  // 取值判空, 默认值规则
  let value = prop ? row[prop] : def;
  if (!value) return value;
  // 人员搜索框: 取值组装为集合
  if (prop.includes("employeeField_")) {
    value = row[prop].reduce((acc, cur, idx) => {
      acc.push({ key: row[`${prop}_id`][idx], label: cur });
      return acc;
    }, []);
  }
  // 附件赋值, jsonString 转镀锡即可
  if (prop.includes("attachmentField_")) {
    value = JSON.parse(row[prop]);
  }
  return value;
}

// 明细组件数据格式化, 接口服务人员搜索框需要集合的 json string
export function __redundancy_server__ (row, prop, fromComp) {
  let value;
  if (fromComp) {
    value = row[prop]; // 3.0 无filedData包装
    value = value && value != null ? value : undefined;
    if (prop.includes("employeeField_")) {
      const arr = value.reduce((acc, cur) => {
        acc.push(cur.key);
        return acc;
      }, []);
      value = JSON.stringify(arr);
    }
    return value;
  }
  // 接口返回数据, 无 fieldData.value
  if (prop.includes("employeeField_")) {
    value = row[`${prop}_id`];
  } else {
    value = row[prop];
  }
  return value;
}

// 查询条件兼容各种组件形式
export function __redundancy_query__ (that, cur) {
  let value = "cur" in cur ? that.$(cur.cur).getValue() || cur.def : cur.def;
  // 兼容人员搜索框, 单选 + 多选
  if (cur.cur && cur.cur.includes("employeeField_")) {
    const arrEmp = [];
    const state = that.$(cur.cur).getValue() || [];
    // 单选
    if (state.length === undefined) {
      arrEmp.push(state.value);
    }
    // 多选且有值
    if (state.length) {
      arrEmp.push(...state.map((emp) => emp.value));
    }
    value = arrEmp;
    // 兼容匹配isAll, 避免被忽略
    if (!value.length) value = undefined;
  }
  // 兼容日期查询: 时间戳, cur为开始, end为默认, 若无则传def
  if (cur.src.includes("dateField_")) {
    const end = "end" in cur ? that.$(cur.end).getValue() || cur.def : cur.def;
    value = [value, end];
    // 兼容匹配isAll, 避免被忽略
    if (!value || !end) value = undefined;
  }
  // 兼容部门, 精确匹配, 兼容多选
  if (cur.cur && cur.cur.includes("departmentSelectField_")) {
    const depart = that.$(cur.cur).getValue();
    // 兼容匹配isAll, 避免被忽略
    if (!depart.length) {
      value = undefined;
    } else {
      value = depart.map((item) => item.value).join(",");
    }
  }
  // 20.8.20 关联表单
  if (cur.cur && cur.cur.includes("associationFormField_") && value.length) {
    value = value.shift().title;
  }
  return value;
}
