import {
  showMessage,
  showLoading,
  hideLoading,
  _resetDetailData,
  __redundancy_value__,
} from "./aliwork-v3";

////////////////////////////////////////////////  逻辑组装  ////////////////////////////////////////////////

/** @exports 明细数据子表全量赋值 */
export function updateDetailTable (that, { details, props, compId_detail_cur, funcFilter } = {}) {
  _resetDetailData(that, compId_detail_cur, props); // 重置明细数据
  if (!details.length) return;
  if (funcFilter) {
    // 过滤方法条件：detail => detail["compId"] == "条件"
    details = details.filter((detail) => funcFilter(detail));
  }
  details = details.map((detail) => {
    return props.reduce((row, prop) => {
      row[prop.cur] = __redundancy_value__(detail, prop.src, prop.def);
      return row;
    }, {});
  });
  that.$(compId_detail_cur).setValue(details);
  return details;
}

////////////////////////////////////////////////  接口服务  ////////////////////////////////////////////////

/** @exports 明细数据全量读取：前端接口 */
// 新建远程请求: 填写地址: https://www.aliwork.com/alibaba/web/APP_CVW86QLT3Q201DLTUIEY/v1/form/listTableDataByFormInstIdAndTableId.json, 更新appType, 关闭自动加载
export async function queryDetailTable (that,
  { dpRemote = "getDetailList", currentPage = 1, pageSize = 50, formUuid, formInstId, compId_detail_src, } = {},
  { details = [], isLoading = true, hideTip, limit = 500 } = {}) {
  if (isLoading) showLoading(that);
  // 实现: 数据缓存
  const res = await that.dataSourceMap[dpRemote]
    .load({ formUuid, formInstanceId: formInstId, tableFieldId: compId_detail_src, currentPage, pageSize, })
    .catch((error) => {
      if (!hideTip) showMessage(that, error.message, "error");
    });
  if (!res) return details;
  details.push(...res.data);
  // 递归: 等待异步
  if (details.length < limit && details.length < res.totalCount) {
    await queryDetailTable(that, { dpRemote, currentPage: res.currentPage + 1, pageSize, formUuid, formInstId, compId_detail_src, },
      { hideTip, details, isLoading, limit, });
  }
  if (isLoading) hideLoading();
  return details;
}

/** @exports 明细数据全量读取与更新：前端实现 */
export async function queryDetailTableAndUpdate (that, { dpRemote, currentPage, pageSize, formUuid, resList, compId_detail_src, props, compId_detail_cur, funcFilter, } = {},
  { limit, hideTip, isLoading, details } = {}) {
  const form = resList[0];
  if (!form) return;
  const table = await queryDetailTable(that, { dpRemote, currentPage, pageSize, hideTip, formUuid, formInstId: form.formInstId, compId_detail_src, },
    { hideTip, details, isLoading, limit, });
  return updateDetailTable(that, { details: table, props, compId_detail_cur, funcFilter, });
}
