/** @内部API */
import {
  aliworkTheDynamicSubsidiary,
  aliworkTheDynamicInsert,
  aliworkTheDynamicDelete,
  aliworkTheDynamicUpdate,
  aliworkTheDynamicDetail,
} from "../service/network";

//////////////////////////////////////////////// 冗余封装 ////////////////////////////////////////////////

// 冗余Toast提示方法
export function _showMessage (msg, type = "success") {
  if (!msg) return;
  const ctx = window.LeGao.getContext();
  ctx.fn.toast({
    title: msg,
    type,
  });
}

// 冗余显示全屏loading: 全局对象
let G_TOAST;
export function _showLoading (msg) {
  const ctx = window.LeGao.getContext();
  G_TOAST = ctx.fn.toast({
    type: "mask_loading", // 支持 loading/info/success/error/nw_loading(3.3.0 支持)/mask_loading(3.4.1 支持)
    title: msg || "加载中...",
  });
}

// 冗余隐藏全屏loading: 全局对象
export function _hideLoading () {
  G_TOAST && G_TOAST();
  G_TOAST = null;
}

// 检测页面状态 : 非提交态, 详情态(单据详情页编辑态 & 流程审批态)
export function _checkSubmitEnv (ctx = window.LeGao.getContext()) {
  const modeData = ctx.getInstanceData().flowData || {};
  return !(modeData.viewMode || modeData.editMode);
}

// 弹出确认框
export async function _showConfirm (title, message, method = "confirm") {
  const ctx = window.LeGao.getContext();
  return new Promise((resolve) => {
    ctx.fn.dialog({
      method,
      title,
      content: message,
      callback: (flag) => {
        if (flag) resolve();
      },
    });
  });
}

// 冗余时间格式化
export function _formatDate (date = new Date(), fmt = "yyyy-MM-dd HH:mm:ss") {
  ctx.fn.formatter("date", date, fmt);
}

// 格式化实例ID跳转详情地址
export function _formatInstanceLink ({
  appType = pageConfig.appType,
  formInstId,
  formUuid,
}) {
  return `https://www.aliwork.com/alibaba/web/${appType}/inst/formEdit.html?formInstId=${formInstId}&formUuid=${formUuid}`;
}

// 获取当前地址的实例ID
export function _valueInstanceIdHref () {
  return window.location.href.split("formInstId=")[1].split("&")[0];
}

// 触发免登后跳转
export function _redirectTask () {
  // 格式为当前页面免登地址 + &formList= + 目标表的FormId：FORM-xxxx
  const formList = window.location.href.split("&formList=");
  if (formList.length != 2) return;
  const appType = pageConfig.appType;
  const url = formList[1].replace("#/", "").replace("&", "");
  window.location.href = `https://www.aliwork.com/alibaba/web/${appType}/inst/homepage/#/${url}`; // 手机端会自动添加 &
}

// NOTE: 联动绑定逻辑: 目前手动通过 JS 赋值的方式，再触发联动还需要重新触发联动
export function _triggerFieldLinkage (formFieldLinkageList = []) {
  const ctx = window.LeGao.getContext(); // 读取全局才有效
  // 通过远程接口获取数据，并赋值给得分组件
  formFieldLinkageList.forEach((fieldId) => triggerFieldLinkage(ctx, fieldId));
  const globalEvents = ctx.getGlobalEvents();
  const linkageMap = globalEvents[fieldId] || {};
  const linkageEvents = linkageMap["onChange"] || [];
  linkageEvents.forEach((event) => {
    ctx.params = {
      ...event.params,
    };
    typeof event.func === "function" && event.func(ctx);
  });
}

// NOTE: [非定时器方案]通过 emitter.on 等触发的回调中，如果有操作明细组件，需要补充明细禁用和启用限制。否则会出现联动后，输入框值无效，无法录入情况 - https://www.yuque.com/yida/help/aagb7a#700a39bc
export function _emitterCompCallBack (compId, changeCallBack) {
  const _context = window.LeGao && window.LeGao.getContext();
  _context &&
    _context.emitter.on(compId, (params = {}) => {
      changeCallBack && changeCallBack(_context, params);
    });
}

////////////////////////////////////////////////  私有方法  ////////////////////////////////////////////////

// 接口返回取值到组件, 人员搜索框需要对象类型
function __redundancy_value__ (row, prop, def) {
  if (!prop) return def;
  // 取值判空, 默认值规则
  let value = prop ? row[prop] : def;
  if (!value) return value;
  // 人员搜索框: 取值组装为集合
  if (prop.includes("employeeField")) {
    value = row[prop].reduce((acc, cur, idx) => {
      acc.push({ key: row[`${prop}_id`][idx], label: cur });
      return acc;
    }, []);
  }
  // 附件赋值, jsonString 转镀锡即可
  if (prop.includes("attachmentField")) {
    value = JSON.parse(row[prop]);
  }
  return value;
}

// 明细组件数据格式化, 接口服务人员搜索框需要集合的 json string
function __redundancy_server__ (row, prop, fromComp) {
  let value;
  if (fromComp) {
    value = ((row[prop] || {}).fieldData || {}).value;
    value = value && value != null ? value : undefined;
    if (prop.includes("employeeField")) {
      const arr = value.reduce((acc, cur) => {
        acc.push(cur.key);
        return acc;
      }, []);
      value = JSON.stringify(arr);
    }
    return value;
  }
  // 接口返回数据, 无 fieldData.value
  if (prop.includes("employeeField")) {
    value = row[`${prop}_id`];
  } else {
    value = row[prop];
  }
  return value;
}

////////////////////////////////////////////////  逻辑组装  ////////////////////////////////////////////////

// 赋值明细组件: 通过 emitter.on 等触发的回调中，处理详见 _emitterCompCallBack
export function _updateCompDetail (ctx, compId_detail, details) {
  const compDetail = ctx.store.get(compId_detail);
  // NOTE: 宜搭2.0明细赋值，禁用其状态，避免赋值后触发了组件的数据联动，导致手动修改数据无效
  if (details.length) compDetail.setCurrentTargetDisable(true);
  setTimeout(() => {
    // 赋值不能及时更新: 集合为空执行报错
    compDetail.mergeVal(details);
    compDetail.setCurrentTargetDisable(false);
  });
  return details;
}

// 重置明细数据
export function _resetDetailData (ctx, compId_detail, props, retainOne = false) {
  // 兼容保留一条清空, 因为 rest() 有 BUG, 在移动端数据关联会失效
  const compDetail = ctx.store.get(compId_detail);
  let details = compDetail.getVal();
  if (retainOne) details = [details.shift];
  // 兼容不传入props, 自动组装
  if (!props) {
    details.forEach((detail) =>
      Object.keys(detail).forEach((key) => {
        detail[key] = { fieldData: { value: "", text: "" } }; // 明细空白解决办法
      })
    );
  } else {
    details.forEach((detail) =>
      props.forEach((prop) => {
        detail[prop.cur] = { fieldData: { value: prop.def, text: prop.def } }; // 明细空白解决办法
      })
    );
  }
  return _updateCompDetail(ctx, compId_detail, details);
}

// 明细更新明细数据
export function _updateCompDetailFromDetail (
  ctx,
  { resList, props, compId_detail_src, compId_detail_cur },
  { funcFilter, isForm = true }
) {
  _resetDetailData(ctx, compId_detail_cur, props); // 重置明细数据
  if (!resList.length) return;
  const form = resList[0][isForm ? "formData" : "data"];
  if (!form) return;
  let srcDetails = form[compId_detail_src];
  if (funcFilter) {
    // 过滤方法条件：detail => detail["compId"] == "条件"
    srcDetails = srcDetails.filter((detail) => funcFilter(detail));
  }
  const details = srcDetails.map((detail) => {
    return props.reduce((row, prop) => {
      const value = __redundancy_value__(detail, prop.src, prop.def);
      row[prop.cur] = { fieldData: { value, text: value } }; // 明细空白解决办法
      return row;
    }, {});
  });
  return _updateCompDetail(ctx, compId_detail_cur, details);
}

// 主表更新主表数据: 兼容一些当前页面需要公式或者数据联动情况 ➜ 若需要作为联动条件, 详见: _triggerFieldLinkage
export function _updateCompMainFromMain (
  ctx,
  { resList, mains },
  { isForm = true } = {}
) {
  const isReset = !resList.length;
  const form = isReset ? {} : resList[0][isForm ? "formData" : "data"];
  mains.forEach((prop) => {
    const value = __redundancy_value__(form, prop.src, prop.def);
    ctx.store.get(prop.cur).setVal(value);
  });
}

// 明细加载主表数据
export async function _updateCompDetailFromMain (
  ctx,
  { resList, props, compId_detail_cur },
  { funcFilter, isForm = true }
) {
  _resetDetailData(ctx, compId_detail_cur, props); // 重置明细数据
  if (!resList.length) return;
  let srcList = resList;
  if (funcFilter) {
    // 过滤方法条件：main => main["compId"] == "条件"
    srcList = srcList.filter((item) =>
      funcFilter(item[isForm ? "formData" : "data"])
    );
  }
  const details = srcList.map((item) => {
    const detail = item[isForm ? "formData" : "data"];
    return props.reduce((row, prop) => {
      let value = __redundancy_value__(detail, prop.src, prop.def);
      // 兼容主表填充明细获取实例ID
      if (prop.def == "formInstId") value = item[prop.def];
      row[prop.cur] = { fieldData: { value, text: value } }; // 明细空白解决办法
      return row;
    }, {});
  });
  return _updateCompDetail(ctx, compId_detail_cur, details);
}

// 人员搜索框格式为 json string: 变更记录, 支持多选, 支持返回值
// mjs._assembleCompForKeyValue(ctx, { compId_get: "需要被格式化的组件id", compId_set: "格式化后储存的组件id" })
export function _assembleCompForKeyValue (ctx, { compId_get, compId_set }) {
  if (!compId_get) return;
  let value = ctx.store.get(compId_get).getVal();
  if (compId_get.includes("employee")) {
    // 人员搜索框: 转为 json string, 宜搭服务会转义 [] 字符, 需要提前拼接存放于组件
    value = value.reduce((acc, cur) => {
      acc.push(cur.key);
      return acc;
    }, []);
    const arrJson = JSON.stringify(value);
    if (compId_set) ctx.store.get(compId_set).setVal(arrJson);
    return arrJson;
  }
  return value;
}

// 明细组件转数据源
// [{ 组件Id: value }] 格式 - 服务明细字段需要提前拼接，全量覆盖  -- 明细转 jsonStr 后, 服务内的组件取值不要加双引号
export function _assembleDetailForKeyValue (
  ctx,
  { compId_detail, compId_textArea },
  details
) {
  // 服务: 明细覆盖, 无需更新明细内的组件 id, 需要确保变更的明细组件和原始明细组件是一致的  -- 明细内有需要同步数据点组件都要绑定这个方法
  if (!details) details = ctx.store.get(compId_detail).getVal() || [];
  const arrData = details.reduce((acc, cur) => {
    const sub = Object.keys(cur).reduce((obj, prop) => {
      const value = __redundancy_server__(cur, prop, true);
      obj[prop] = value;
      return obj;
    }, {});
    delete sub["__mergeAfterBind"];
    acc.push(sub);
    return acc;
  }, []);
  const jsonStr = JSON.stringify(arrData);
  if (compId_textArea) ctx.store.get(compId_textArea).setVal(jsonStr);
  return jsonStr;
}

// 按需取明细组件
// props = [{compId: "", field: ""}] -- 组件ID和接口需要字段名
export function _formatDetailToJsonString (ctx, { compId_detail, compId_textArea }, props) {
  if (!compId_detail) {
    ctx.store.get(compId_textArea).setVal(JSON.stringify(props));
    return props;
  }
  const details = ctx.store.get(compId_detail).getVal();
  const data = details.reduce((acc, cur) => {
    acc.push(
      props.reduce((obj, item) => {
        const value = __redundancy_server__(cur, item.compId, true);
        obj[item.field] = value;
        return obj;
      }, {})
    );
    return acc;
  }, []);
  if (compId_textArea) ctx.store.get(compId_textArea).setVal(JSON.stringify(data));
  return data;
}

////////////////////////////////////////////////  接口服务  ////////////////////////////////////////////////

// 查询表单或者流程封装
// const conditions = [{ src: "", cur: "", def: "" }];  // def 优于 src, src 和 cur 均需要传入组件ID
export async function _queryFormOrProcess (
  ctx,
  { formId, conditions = [], compId_tip } = {},
  { isForm = true, errMsg = "未查询到匹配数据", isAll = true, isHide } = {},
  { pageIndex, pageSize, queryAll } = {}
) {
  // 条件格式化: val 兼容多选
  const conditionList = conditions.reduce((acc, cur) => {
    const value =
      "cur" in cur ? ctx.store.get(cur.cur).getVal() || cur.def : cur.def;
    if (value) {
      // def 优于 src, src 和 cur 均需要传入组件ID
      acc.push({
        fieldName: cur.src,
        value,
      });
    }
    return acc;
  }, []);
  if (conditions.length && !conditionList.length) return [];
  // 是否强制匹配多参必须都有值
  if (isAll && conditionList.length != conditions.length) return [];
  // 处理接口全局公共参数
  const queryUrl = isForm ? "/form/select" : "/process/select";
  const res = await aliworkTheDynamicSubsidiary(queryUrl, {
    ...window.zParams,
    formId,
    conditionList,
    pageIndex,
    pageSize,
  }).catch((err) => err);
  const isErr = res.list && res.list.length;
  // 容错 && 提示
  if (res.message) errMsg = res.message;
  if (compId_tip) {
    const compState = ctx.store.get(compId_tip);
    compState.invokeValidate(isErr, errMsg);
    compState.forceValid();
  } else {
    if (!isHide && !isErr) showMessage(that, errMsg, "error");
  }
  // TODO: queryAll: 递归查询主表宜搭限制为100每次分页上限, 兼容累计查询结束
  return res.list || [];
}
_updateCompDetail;
// 值多选查询表单或者流程: 多选条件仅仅支持一个字段; 以第一条查询为主数据, 合并明细; 数据联动可能会异常
export async function _queryMultipleFormOrProcess (
  ctx,
  { formId, conditions = [], compId_tip, compId_detail_src } = {},
  { isForm, errMsg, isHide, isAll } = {}
) {
  const multiple = conditions.find((item) =>
    item.cur.includes("multiSelectField")
  );
  const condition = ctx.store.get(multiple.cur).getVal();
  if (!condition.length) {
    multiple.val = "";
    return await _queryFormOrProcess(
      ctx,
      { formId, conditions, compId_tip },
      { isForm, errMsg, isHide, isAll }
    );
  } else {
    // NOTE: 当 async/await 遇上 forEach: https://www.jianshu.com/p/4ec6a55c26d7
    const result = [];
    for (let val of condition) {
      multiple.val = val;
      const resList = await _queryFormOrProcess(
        ctx,
        { formId, conditions, compId_tip },
        { isForm, errMsg, isHide, isAll }
      );
      // FIXME: 将主表数据合并到子表, 合并后删除掉主表的子表数据
      resList.forEach((item) => {
        const formData = item.formData;
        const details = formData[compId_detail_src].map((detail) => {
          delete formData[compId_detail_src];
          return {
            ...detail,
            ...formData,
          };
        });
        result.push(...details);
      });
    }
    return result;
  }
}

// 更新当前表单的字段
// const updateList = [{ fieldName: compId, value: changeValue }]; // 更新内容对照表
export async function _updateFormList (
  { updateList, message = "更新", formInstId },
  { isReload = false }
) {
  if (!formInstId) formInstId = _valueInstanceIdHref();
  // 处理接口全局公共参数: userId 接口有权限管理
  const res = await aliworkTheDynamicUpdate("/form/update", {
    ...window.zParams,
    formInstId,
    updateList,
  }).catch(() => []);
  const isSuccess = res.success;
  _showMessage(
    message + (isSuccess ? "成功" : "失败"),
    isSuccess ? "success" : "error"
  );
  if (isReload && isSuccess) window.location.reload();
}

// 新增表单记录: 若通过拷贝形式新增, 两表组件id必须一一对应(formInstId) ➜ 多字段不会写入
export async function _insertFromInstance (
  ctx,
  { formUuid, props, formInstId },
  formatParams = (params) => params
) {
  let formDataJson = {};
  if (formInstId) {
    const resp = await aliworkTheDynamicDetail("/form/getFormDataById", {
      ...zParams,
      formInstId,
    }).catch(() => { });
    formDataJson = Object.keys(resp.obj).reduce((acc, prop) => {
      const value = __redundancy_server__(resp.obj, prop);
      acc[prop] = value;
      return acc;
    }, {});
  } else {
    // 不支持明细数据
    formDataJson = props.reduce((acc, cur) => {
      const value = _assembleCompForKeyValue(ctx, { compId_get: cur.cur });
      acc[cur.src] = value || cur.def;
      return acc;
    }, {});
  }
  // 参数回调修改机制
  formDataJson = (formatParams && formatParams(formDataJson)) || formDataJson;
  const res = await aliworkTheDynamicInsert("/form/insert", {
    ...window.zParams,
    formUuid,
    formDataJson: JSON.stringify(formDataJson),
  }).catch((err) => err);
  const isSuccess = res.success;
  _showMessage(
    isSuccess ? "强开成功" : res.message,
    isSuccess ? "success" : "error"
  );
  return res;
}

// 删除表单实例
export async function _deleteFormInstance ({ formInstId }, callBack) {
  if (!formInstId) formInstId = _valueInstanceIdHref();
  const res = await aliworkTheDynamicDelete("/form/delete", {
    ...window.zParams,
    formInstId,
  }).catch((err) => err);
  const isSuccess = res.success;
  _showMessage(
    isSuccess ? "抢单成功" : res.message,
    isSuccess ? "success" : "error"
  );
  callBack && callBack(isSuccess);
}
