/** @网络请求API */
import {
  aliworkTheDynamicSubsidiary,
  aliworkTheDynamicInsert,
  aliworkTheDynamicDelete,
  aliworkTheDynamicUpdate,
  aliworkTheDynamicDetail,
} from "../service/network";

/** @工具库API */
import { debounce } from "../utils/optimize";

//////////////////////////////////////////////// 冗余封装 ////////////////////////////////////////////////

// 冗余Toast提示方法
export function showMessage (that, title, type = "success") {
  if (!title) return;
  that.utils.toast({ type, title });
}

// 冗余显示全屏loading: 全局对象
let G_DIALOG;
export function showLoading (that, title = "拼命加载中...") {
  if (G_DIALOG) return;
  G_DIALOG = that.utils.toast({
    type: "loading",
    title,
    closeable: false,
    footer: false,
    messageProps: {
      type: "loading",
    },
  });
}

// 冗余隐藏全屏loading: 全局对象
export function hideLoading () {
  G_DIALOG && G_DIALOG();
  G_DIALOG = null;
}

// 检测页面状态: 提交态都为undefined; 详情页viewMode为true, editMode为false; 编辑和流程节点editMode为true, viewMode为false
export function checkSubmitEnv (that) {
  // 枚举: 0提交，1查看，2编辑，-1自定义页面
  const instanceData = that.utils.getFormInstanceData();
  if (!instanceData) return -1; // 自定义页面
  const { flowData = {} } = instanceData;
  const { editMode, viewMode } = flowData;
  if (editMode) return 2; // 审批页面为2, 编辑状态
  if (viewMode) return 1;
  return 0;
}

// 弹出确认框
export async function showConfirm (that, title, message, method = "confirm") {
  return new Promise((resolve) => {
    that.fn.dialog({
      method,
      title,
      content: message,
      callback: (flag) => {
        if (flag) resolve();
      },
    });
  });
}

// 冗余时间格式化
export function formatDate (
  that,
  date = new Date(),
  fmt = "yyyy-MM-dd HH:mm:ss"
) {
  that.utils.formatter(type, date, fmt);
}

// 格式化实例ID跳转详情地址
export function formatInstanceLink ({
  appType = pageConfig.appType,
  formInstId,
  formUuid,
}) {
  return `https://www.aliwork.com/alibaba/web/${appType}/inst/formEdit.html?formInstId=${formInstId}&formUuid=${formUuid}`;
}

// 获取当前地址的实例ID
export function valueInstanceId (that) {
  if (that) return that.utils.router.getQuery().formInstId;
  return window.location.href.split("formInstId=")[1].split("&")[0];
}

// 触发免登后跳转
export function redirectTaskLink () {
  // 格式为当前页面免登地址 + &formList= + 目标表的FormId：FORM-xxxx
  const formList = window.location.href.split("&formList=");
  if (formList.length != 2) return;
  const appType = pageConfig.appType;
  const url = formList[1].replace("#/", "").replace("&", "");
  window.location.href = `https://www.aliwork.com/alibaba/web/${appType}/inst/homepage/#/${url}`; // 手机端会自动添加 &
}

////////////////////////////////////////////////  私有方法  ////////////////////////////////////////////////

// 接口返回取值到组件, 人员搜索框需要对象类型
export function __redundancy_value__ (row, prop, def) {
  // 为空返回默认值
  if (!prop) return def;
  // 取值判空, 默认值规则
  let value = prop ? row[prop] : def;
  if (!value) return value;
  // 人员搜索框: 取值组装为集合
  if (prop.includes("employeeField")) {
    value = row[prop].reduce((acc, cur, idx) => {
      acc.push({ key: row[`${prop}_id`][idx], label: cur });
      return acc;
    }, []);
  }
  // 附件赋值, jsonString 转镀锡即可
  if (prop.includes("attachmentField")) {
    value = JSON.parse(row[prop]);
  }
  return value;
}

// 明细组件数据格式化, 接口服务人员搜索框需要集合的 json string
export function __redundancy_server__ (row, prop, fromComp) {
  let value;
  if (fromComp) {
    value = row[prop]; // 3.0 无filedData包装
    value = value && value != null ? value : undefined;
    if (prop.includes("employeeField")) {
      const arr = value.reduce((acc, cur) => {
        acc.push(cur.key);
        return acc;
      }, []);
      value = JSON.stringify(arr);
    }
    return value;
  }
  // 接口返回数据, 无 fieldData.value
  if (prop.includes("employeeField")) {
    value = row[`${prop}_id`];
  } else {
    value = row[prop];
  }
  return value;
}

// 查询条件兼容各种组件形式
export function __redundancy_query__ (that, cur) {
  let value = "cur" in cur ? that.$(cur.cur).getValue() || cur.def : cur.def;
  // 兼容人员搜索框, 单选 + 多选
  if (cur.cur && cur.cur.includes("employeeField")) {
    const arrEmp = [];
    const state = that.$(cur.cur).getValue() || [];
    // 单选
    if (state.length === undefined) {
      arrEmp.push(state.value);
    }
    // 多选且有值
    if (state.length) {
      arrEmp.push(...state.map((emp) => emp.value));
    }
    value = arrEmp;
    // 兼容匹配isAll, 避免被忽略
    if (!value.length) value = undefined;
  }
  // 兼容日期查询: 时间戳, cur为开始, end为默认, 若无则传def
  if (cur.src.includes("dateField")) {
    const end = "end" in cur ? that.$(cur.end).getValue() || cur.def : cur.def;
    value = [value, end];
    // 兼容匹配isAll, 避免被忽略
    if (!value || !end) value = undefined;
  }
  // 兼容部门, 精确匹配, 兼容多选
  if (cur.cur && cur.cur.includes("departmentSelectField")) {
    const depart = that.$(cur.cur).getValue();
    // 兼容匹配isAll, 避免被忽略
    if (!depart.length) {
      value = undefined;
    } else {
      value = depart.map((item) => item.value).join(",");
    }
  }
  // 20.8.20 关联表单
  if (cur.cur && cur.cur.includes("associationFormField") && value.length) {
    value = value.shift().title;
  }
  return value;
}

// 重置明细数据
export function _resetDetailData (
  that,
  compId_detail,
  props,
  retainOne = false
) {
  // 兼容保留一条清空, 因为 rest() 有 BUG, 在移动端数据关联会失效
  const compDetail = that.$(compId_detail);
  let details = compDetail.getValue();
  if (retainOne) details = [details.shift];
  // 兼容不传入props, 自动组装
  if (!props) {
    details.forEach((detail) =>
      Object.keys(detail).forEach((key) => {
        detail[key] = "";
      })
    );
  } else {
    details.forEach((detail) =>
      props.forEach((prop) => {
        detail[prop.cur] = prop.def;
      })
    );
  }
  compDetail.setValue(details);
}

////////////////////////////////////////////////  逻辑组装  ////////////////////////////////////////////////

// 明细更新明细数据
export function updateCompDetailFromDetail (
  that,
  { resList, props, compId_detail_src, compId_detail_cur },
  { funcFilter, isForm = true, isUpdate = true }
) {
  _resetDetailData(that, compId_detail_cur, props); // 重置明细数据
  if (!resList.length) return;
  const form = resList[0][isForm ? "formData" : "data"];
  if (!form) return;
  let srcDetails = form[compId_detail_src];
  if (funcFilter) {
    // 过滤方法条件：detail => detail["compId"] == "条件"
    srcDetails = srcDetails.filter((detail) => funcFilter(detail));
  }
  const details = srcDetails.map((detail) => {
    return props.reduce((row, prop) => {
      row[prop.cur] = __redundancy_value__(detail, prop.src, prop.def);
      return row;
    }, {});
  });
  /** 连续更新会异常: 需要调用调用forceUpdate才会生效, 若后续即将有更新, 这里置为false, 匹配明细: queryArrayFormOrProcessAndUpdate */
  if (isUpdate) that.$(compId_detail_cur).setValue(details);
  return details;
}

// 主表更新主表数据: 兼容一些当前页面需要公式或者数据联动情况
export function updateCompMainFromMain (
  that,
  { resList, mains },
  { isForm = true } = {}
) {
  const isReset = !resList.length;
  const form = isReset ? {} : resList[0][isForm ? "formData" : "data"];
  mains.forEach((prop) => {
    const value = __redundancy_value__(form, prop.src, prop.def);
    that.$(prop.cur).setVal(value);
  });
}

// 明细加载主表数据
export async function updateCompDetailFromMain (
  that,
  { resList, props, compId_detail_cur },
  { funcFilter, isForm = true }
) {
  _resetDetailData(that, compId_detail_cur, props); // 重置明细数据
  if (!resList.length) return;
  let srcList = resList;
  if (funcFilter) {
    // 过滤方法条件：main => main["compId"] == "条件"
    srcList = srcList.filter((item) =>
      funcFilter(item[isForm ? "formData" : "data"])
    );
  }
  const details = srcList.map((item) => {
    const detail = item[isForm ? "formData" : "data"];
    return props.reduce((row, prop) => {
      let value = __redundancy_value__(detail, prop.src, prop.def);
      // 兼容主表填充明细获取实例ID
      if (prop.def == "formInstId") value = item[prop.def];
      row[prop.cur] = value;
      return row;
    }, {});
  });
  that.$(compId_detail_cur).setValue(details);
  return details;
}

// 人员搜索框格式为 json string: 变更记录, 支持多选, 支持返回值
// mjs._assembleCompForKeyValue(that, { compId_get: "需要被格式化的组件id", compId_set: "格式化后储存的组件id" })
export function assembleCompForKeyValue (that, { compId_get, compId_set }) {
  if (!compId_get) return;
  let value = that.$(compId_get).getValue();
  if (!!prop && compId_get.includes("employeeField")) {
    // 人员搜索框: 转为 json string, 宜搭服务会转义 [] 字符, 需要提前拼接存放于组件
    value = value.reduce((acc, cur) => {
      acc.push(cur.key);
      return acc;
    }, []);
    const arrJson = JSON.stringify(value);
    if (compId_set) that.$(compId_set).setValue(arrJson);
    return arrJson;
  }
  return value;
}

// 明细组件转数据源
// [{ 组件Id: value }] 格式 - 服务明细字段需要提前拼接，全量覆盖  -- 明细转 jsonStr 后, 服务内的组件取值不要加双引号
export function assembleDetailForKeyValue (
  that,
  { compId_detail, compId_textArea },
  details
) {
  // 服务: 明细覆盖, 无需更新明细内的组件 id, 需要确保变更的明细组件和原始明细组件是一致的  -- 明细内有需要同步数据点组件都要绑定这个方法
  if (!details) details = that.$(compId_detail).getValue() || [];
  const arrData = details.reduce((acc, cur) => {
    const sub = Object.keys(cur).reduce((obj, prop) => {
      const value = __redundancy_server__(cur, prop, true);
      obj[prop] = value;
      return obj;
    }, {});
    acc.push(sub);
    return acc;
  }, []);
  const jsonStr = JSON.stringify(arrData);
  if (compId_textArea) {
    that.$(compId_textArea).setValue(jsonStr);
    return jsonStr;
  }
  return arrData;
}

// 按需取明细组件 - 建议页面非提交态获取，实时获取极不稳定：审批流程页面到到查看状态都是2，编辑。js赋值后同意后数据会保存的，不需要单独调用更新
// props = [{compId: "", field: ""}] -- 组件ID和接口需要字段名
export function formatDetailToJsonString (
  that,
  { compId_detail, compId_textArea },
  props
) {
  if (!compId_detail) {
    that.$(compId_textArea).setValue(JSON.stringify(props));
    return props;
  }
  const details = that.$(compId_detail).getValue();
  const data = details.reduce((acc, cur) => {
    acc.push(
      props.reduce((obj, item) => {
        const value = __redundancy_server__(cur, item.compId, true);
        obj[item.field] = value;
        return obj;
      }, {})
    );
    return acc;
  }, []);
  if (compId_textArea) that.$(compId_textArea).setValue(JSON.stringify(data));
  return data;
}

// 明细序号自增
export function incrementCompDetailID (that, compID_detailed, compID_number) {
  const details = that.$(compID_detailed).getValue();
  details.forEach((item, index) => {
    item[compID_number] = index + 1;
  });
  that.$(compID_detailed).setValue(details);
}

// 更新明细内数据方法回调  -- 若是匹配可以先filter, 然后遍历就行了
export function updateCompDetailFromConfig (
  that,
  { compId_detail },
  funcUpdate
) {
  const compDetail = that.$(compId_detail);
  const details = compDetail.getValue().map((item, idx) => {
    funcUpdate && funcUpdate(item, idx, details);
    return item;
  });
  compDetail.setValue(details);
  return details;
}

// 分类求和: 宜搭明细分类求和，当明细内相同数据选择多次情况，回写中间表使用，总计数更新 [宜搭明细相同条件只会更新最后一条]
export function debounceCalcCategory (
  that,
  { compId_table, funcCondition, compId_cur, compId_sum } = {},
  { funcCondition2, compId_cur2, compId_sum2 } = {}
) {
  if (!that[compId_table]) {
    that[compId_table] = debounce((that) => {
      const compTable = that.$(compId_table);
      const details = compTable.getValue();
      details.forEach((detail) => {
        const value = details.reduce((acc, cur) => {
          if (funcCondition({ cur, detail })) acc += Number(cur[compId_cur]);
          return acc;
        }, 0);
        detail[compId_sum] = value;
        // 多个合并情况
        if (funcCondition2) {
          const value = details.reduce((acc, cur) => {
            if (funcCondition2({ cur, detail }))
              acc += Number(cur[compId_cur2]);
            return acc;
          }, 0);
          detail[compId_sum2] = value;
        }
      });
      compTable.setValue(details);
      compTable.forceUpdate();
    }, 400);
  }
  that[compId_table](that);
}

// 计算流水号, 以 count 公式获取流水号, 设定位数不足部分补 0
export function countSerialNumber (that, compId, length = 5, prefix = "") {
  const num = that.$(compId).getValue();
  const ser = (Array(length).join("0") + num).slice(-length);
  if (prefix) that.$(compId).setValue(`${prefix}${ser}`);
  return ser;
}

////////////////////////////////////////////////  接口服务  ////////////////////////////////////////////////

// 查询表单或者流程封装
// const conditions = [{ src: "", cur: "", def: "" }];  // src和cur需要传入组件ID; cur可不传, 取值def;
export async function queryFormOrProcess (
  that,
  { formId, conditions = [], compId_tip, conditionList = [] } = {},
  { isForm = true, errMsg = "未查询到匹配数据", isHide, isAll } = {},
  { pageIndex, pageSize, queryAll } = {}
) {
  // 查询条件格式化
  if (!conditionList.length) {
    conditionList = conditions.reduce((acc, cur) => {
      const value = __redundancy_query__(that, cur);
      if (value) {
        acc.push({
          fieldName: cur.src,
          value,
        });
      }
      return acc;
    }, []);
    if (conditions.length && !conditionList.length) return [];
    // 是否强制匹配多参必须都有值
    if (isAll && conditionList.length != conditions.length) return [];
  }
  // 处理接口全局公共参数queryFormOrProcess
  const queryUrl = isForm ? "/form/select" : "/process/select";
  const res = await aliworkTheDynamicSubsidiary(queryUrl, {
    ...window.zParams,
    formId,
    conditionList,
    pageIndex,
    pageSize,
  }).catch((err) => err);
  // 容错 && 提示
  const isErr = res.list && res.list.length;
  if (res.message) errMsg = res.message;
  if (compId_tip) {
    const compState = that.$(compId_tip);
    compState.setValidation(
      [
        { type: "required" },
        {
          type: "customValidate",
          message: errMsg,
          param: () => !!isErr,
        },
      ],
      true
    );
  } else {
    if (!isHide && !isErr) showMessage(that, errMsg, "error");
  }
  // TODO: queryAll: 递归查询主表宜搭限制为100每次分页上限, 兼容累计查询结束
  return res.list || [];
}

// 明细内触发查询, 如bom查库存, 当前值传入def: const conditions = [{ src: "", def: "" }];
export async function queryArrayFormOrProcessAndUpdate (
  that,
  { formId, conditions = [], array = [], props = [], compId_detail_cur } = {},
  { isForm = true, isHide = true } = {}
) {
  // NOTE: 当 async/await 遇上 forEach 使用 for ... of 代替: https://www.jianshu.com/p/4ec6a55c26d7
  for (let detail of array) {
    // 入参数格式化
    const conditionList = conditions.reduce((acc, cur) => {
      const value = "cur" in cur ? detail[cur.cur] || cur.def : cur.def;
      acc.push({ fieldName: cur.src, value });
      return acc;
    }, []);
    const resList = await queryFormOrProcess(
      that,
      { formId, conditionList },
      { isForm, isHide }
    );
    if (resList.length) {
      const form = resList[0][isForm ? "formData" : "data"];
      props.forEach((prop) => {
        const value = __redundancy_value__(form, prop.src, prop.def);
        detail[prop.cur] = value;
      });
    }
  }
  /** 连续更新会异常: 需要调用调用forceUpdate才会生效, 若后续即将有更新, 这里置为false, 匹配明细: updateCompDetailFromDetail */
  that.$(compId_detail_cur).setValue(array);
}

// 值多选查询表单或者流程: 多选条件仅仅支持一个字段; 以第一条查询为主数据, 合并明细; 数据联动可能会异常
export async function queryMultipleFormOrProcess (
  that,
  { formId, conditions = [], compId_tip, compId_detail_src } = {},
  { isForm, errMsg, isHide, isAll } = {}
) {
  const multiple = conditions.find((item) =>
    item.cur.includes("multiSelectField")
  );
  const condition = that.$(multiple.cur).getVal();
  if (!condition.length) {
    multiple.val = "";
    return await queryFormOrProcess(
      that,
      { formId, conditions, compId_tip },
      { isForm, errMsg, isHide, isAll }
    );
  } else {
    // NOTE: 当 async/await 遇上 forEach 使用 for ... of 代替: https://www.jianshu.com/p/4ec6a55c26d7
    const result = [];
    for (let val of condition) {
      multiple.val = val;
      const resList = await queryFormOrProcess(
        that,
        { formId, conditions, compId_tip },
        { isForm, errMsg, isHide, isAll }
      );
      // FIXME: 将主表数据合并到子表, 合并后删除掉主表的子表数据
      resList.forEach((item) => {
        const formData = item.formData;
        const details = formData[compId_detail_src].map((detail) => {
          delete formData[compId_detail_src];
          return {
            ...detail,
            ...formData,
          };
        });
        result.push(...details);
      });
    }
    return result;
  }
}

// 更新当前表单的字段
// const updateList = [{ fieldName: compId, value: changeValue }]; // 更新内容对照表
export async function updateFormList (
  that,
  { updateList, message = "更新", formInstId },
  { isReload = false }
) {
  if (!formInstId) formInstId = valueInstanceId(that);
  // 处理接口全局公共参数: userId 接口有权限管理
  const res = await aliworkTheDynamicUpdate("/form/update", {
    ...window.zParams,
    formInstId,
    updateList,
  }).catch(() => []);
  const isSuccess = res.success;
  showMessage(
    that,
    message + (isSuccess ? "成功" : "失败"),
    isSuccess ? "success" : "error"
  );
  if (isReload && isSuccess) window.location.reload();
}

// 新增表单记录: 若通过拷贝形式新增, 两表组件id必须一一对应(formInstId) ➜ 多字段不会写入
export async function insertFromInstance (
  that,
  { formUuid, props, formInstId },
  formatParams = (params) => params
) {
  let formDataJson = {};
  if (formInstId) {
    const resp = await aliworkTheDynamicDetail("/form/getFormDataById", {
      ...zParams,
      formInstId,
    }).catch(() => { });
    formDataJson = Object.keys(resp.obj).reduce((acc, prop) => {
      const value = __redundancy_server__(resp.obj, prop);
      acc[prop] = value;
      return acc;
    }, {});
  } else {
    // 不支持明细数据
    formDataJson = props.reduce((acc, cur) => {
      const value = _assembleCompForKeyValue(that, { compId_get: cur.cur });
      acc[cur.src] = value || cur.def;
      return acc;
    }, {});
  }
  // 参数回调修改机制
  formDataJson = (formatParams && formatParams(formDataJson)) || formDataJson;
  const res = await aliworkTheDynamicInsert("/form/insert", {
    ...window.zParams,
    formUuid,
    formDataJson: JSON.stringify(formDataJson),
  }).catch((err) => err);
  const isSuccess = res.success;
  showMessage(
    that,
    isSuccess ? "强开成功" : res.message,
    isSuccess ? "success" : "error"
  );
  return res;
}

// 删除表单实例
export async function deleteFormInstance (that, { formInstId }, callBack) {
  if (!formInstId) formInstId = valueInstanceId();
  const res = await aliworkTheDynamicDelete("/form/delete", {
    ...window.zParams,
    formInstId,
  }).catch((err) => err);
  const isSuccess = res.success;
  showMessage(
    that,
    isSuccess ? "抢单成功" : res.message,
    isSuccess ? "success" : "error"
  );
  callBack && callBack(isSuccess);
}



