
/*** mjs 之 谷元发票识别 ***/

export default {

  // 修改公共配置
  init () {
    if (process.env.NODE_ENV == "development") {
      mjs.conf.api = "https://mc.cloudpure.cn/api/guyuan";
    } else {
      mjs.conf.api = "https://mc.cloudpure.cn/api/guyuan";
    }
    return this; // this 指向当前项目本身
  },

  // 发票识别, 混票  ppExt: 官方已停止更新, PDF多张异常
  async mixedInvoice (file) {
    const rsp = await mjs.request.xhr.doPost(`${mjs.conf.api}/invoice-iv`, {}, {
      url: file.url,
      isPdf: file.type.includes("pdf"),
      size: file.size / 1024 / 1024
    })
    // 明细数据匹配表头
    const headers = mjs.$this.$("tableField_liv5f4d2").props.children.props.children.map(({ props: { label, fieldId } }) => ({ label, compId: fieldId }))
    const prop = rsp.data.dto; // 通用字段定义
    const invoices = rsp.data.result.map(item => {
      const rowData = Object.keys(prop).reduce((acc, cur) => {
        const comp = headers.find(c => c.label == cur)
        if (comp) {
          acc[comp.compId] = item[prop[cur]]
        }
        // 非标准表头，格式化
        if (cur == "价税合计") {
          acc.numberField_liihyrt7 = item.amount;
        }
        if (cur == "开票日期") {
          acc.textField_livimrja = item[prop[cur]];
          acc.dateField_liihyrt9 = new Date(acc.textField_livimrja).getTime()
        }
        return acc;
      }, {})
      // 发票数据标题: 销售方 + 发票类型 + 价税合计
      rowData.textField_ljmgqvbz = item.sellerName + "-" + item.kindName + "-" + item.amount
      return rowData;
    });
    mjs.$this.$('tableField_liv5f4d2').setValue(invoices);
  },

  // 发票识别, 混票 ppExt: 新版本官方返回数据已结构化, 以官方为准
  async mixedInvoice2 (file) {
    const rsp = await mjs.request.xhr.doPost(`${mjs.conf.api}/invoice-iv2`, {}, {
      url: file.url,
      isPdf: file.type.includes("pdf"),
      size: file.size / 1024 / 1024
    })
    // 明细数据匹配表头
    const headers = mjs.$this.$("tableField_liv5f4d2").props.children.props.children.map(({ props: { label, fieldId } }) => ({ label, compId: fieldId }))
    const prop = rsp.data.dto; // 通用字段定义
    //合计数据
    let amount = 0, tax = 0, exTax = 0;
    const invoices = rsp.data.result.map(item => {
      const rowData = Object.keys(prop).reduce((acc, cur) => {
        const comp = headers.find(c => c.label == cur)
        if (comp) {
          acc[comp.compId] = item[prop[cur]]
        }
        // 非标准表头，格式化
        if (cur == "价税合计") {
          acc.numberField_liihyrt7 = item.amount;
        }
        if (cur == "开票日期") {
          acc.textField_livimrja = item[prop[cur]];
          acc.dateField_liihyrt9 = new Date(acc.textField_livimrja).getTime()
        }
        return acc;
      }, {})
      // 合计数据
      amount += item.amount;
      tax += item.tax;
      exTax += item.excludingTax;
      // 发票数据标题: 销售方 + 发票类型 + 价税合计
      rowData.textField_ljmgqvbz = item.sellerName + "-" + item.kindName + "-" + item.amount
      return rowData;
    });
    mjs.$this.$('numberField_ln05ra1i').setValue(amount);
    mjs.$this.$('numberField_ln05ra1n').setValue(exTax);
    mjs.$this.$('numberField_ln05ra1o').setValue(tax);
    mjs.$this.$('tableField_liv5f4d2').setValue(invoices);
    return invoices;
  },

  // 发票识别, prd 10.30 调整为多选识别 [同时选择多个文件，触发多次回调，全量查询方案: 防抖调用]
  async mixedInvoice3 (files) {
    let prop = {} // 通用字段定义
    const results = []
    for (const file of files) {
      console.log(file)
      const rsp = await mjs.request.xhr.doPost(`${mjs.conf.api}/invoice-iv2`, {}, {
        url: file.url,
        isPdf: file.name.includes("pdf"),
        size: file.size / 1024 / 1024
      })
      prop = rsp.data.dto;
      results.push(...rsp.data.result);
    }
    // 明细数据匹配表头 [二次获取无效]
    if (!this.headers) {
      this.headers = mjs.$this.$("tableField_liv5f4d2").props.children.props.children.map(({ props: { label, fieldId } }) => ({ label, compId: fieldId }))
    }
    //合计数据
    let amount = 0, tax = 0, exTax = 0;
    const invoices = results.map(item => {
      const rowData = Object.keys(prop).reduce((acc, cur) => {
        const comp = this.headers.find(c => c.label == cur)
        if (comp) {
          acc[comp.compId] = item[prop[cur]]
        }
        // 非标准表头，格式化
        if (cur == "价税合计") {
          acc.numberField_liihyrt7 = item.amount;
        }
        if (cur == "开票日期") {
          acc.textField_livimrja = item[prop[cur]];
          acc.dateField_liihyrt9 = new Date(acc.textField_livimrja).getTime()
        }
        return acc;
      }, {})
      // 合计数据
      amount += item.amount;
      tax += item.tax;
      exTax += item.excludingTax;
      // 发票数据标题: 销售方 + 发票类型 + 价税合计
      rowData.textField_ljmgqvbz = item.sellerName + "-" + item.kindName + "-" + item.amount
      rowData.radioField_loc7gxul = "否";
      return rowData;
    });
    mjs.$this.$('numberField_ln05ra1i').setValue(amount);
    mjs.$this.$('numberField_ln05ra1n').setValue(exTax);
    mjs.$this.$('numberField_ln05ra1o').setValue(tax);
    mjs.$this.$('tableField_liv5f4d2').setValue(invoices);
    return invoices;
  },

  // 批量验证
  batchCheck () {
    return new Promise((resolve, reject) => {
      const details = mjs.$this.$('tableField_liv5f4d2').getValue()
      const param = details.map(item => {
        return {
          // ppExt: 全电票, 新版本识别接口, 返回名称为: 电子发票(普通发票) 不包含全电标识, 发类型为: 全电发票. 注意取值
          name: item.textField_liwbaczr,
          kindName: item.selectField_liihyrta,
          serial: item.textField_liihyrt8,
          code: item.textField_lil34mnc,
          date: item.textField_livimrja,
          amount: item.numberField_liihyrt7,
          excludingTax: item.numberField_livimrj8,
          checkCode: item.textField_lil34mne,
          buyerName: item.selectField_lix0cyqx,
          buyerTaxId: item.textField_lil34mng,
          sellerName: item.textField_liihyrt3,
          sellerTaxId: item.textField_lil34mnf,
        }
      })
      mjs.request.xhr.doPost(`${mjs.conf.api}/invoice-va`, {}, {
        param
      }).then(() => resolve()).catch(err => {
        if (err.includes("【")) {
          const s = err.split("】")[0]
          const e = s.split("【")[1]
          for (let i = 1; i < e; i++) {
            details[i - 1].radioField_loc7gxul = "是"
          }
          mjs.$this.$('tableField_liv5f4d2').setValue(details)
        }
        reject()
      }) // ppExt: 需要拦截错误, 否则一次失败后程序阻断
    })
  },

  // 发票查重, 验真
  validate () {
    return new Promise((resolve, reject) => {
      if (mjs.env == 2) {
        if (mjs.$this.$('selectField_liihyrt6').getValue() == "已使用") {
          return mjs.com.toastError("发票已被使用，不允许修改！")
        }
        // 校验通过: 发票未使用状态且不在流程中 || 否改为否，标识状态不做校验
        if (mjs.$this.$('radioField_liihyrtb').getValue() == "是" || mjs.corp.guyuan.status == "否") {
          return mjs.com.toastError(null, resolve)
        }
      }
      // 非作废校验
      const param = {
        // ppExt: 全电票, 新版本识别接口, 返回名称为: 电子发票(普通发票) 不包含全电标识, 发类型为: 全电发票. 注意取值
        name: mjs.$this.$("textField_ljasce9j").getValue(),
        kindName: mjs.$this.$("selectField_liihyrta").getValue(),
        serial: mjs.$this.$("textField_liihyrt8").getValue(),
        code: mjs.$this.$("textField_lil34mnc").getValue(),
        date: mjs.$this.$("textField_liwadykg").getValue(),
        amount: mjs.$this.$("numberField_liihyrt7").getValue(),
        excludingTax: mjs.$this.$("numberField_liwauxaq").getValue(),
        checkCode: mjs.$this.$("textField_lil34mne").getValue(),
        buyerName: mjs.$this.$('selectField_lix0cyqx').getValue(),
        buyerTaxId: mjs.$this.$('textField_lil34mng').getValue(),
        sellerName: mjs.$this.$('textField_liihyrt3').getValue(),
        sellerTaxId: mjs.$this.$('textField_lil34mnf').getValue(),
      };
      mjs.request.xhr.doPost(`${mjs.conf.api}/invoice-va`, {}, {
        param: [param]
      }).then(() => resolve()).catch(err => reject(err)) // ppExt: 需要拦截错误, 否则一次失败后程序阻断
    })
  },

  // 提交校验 [日常报销/项目付款]
  submit () {
    const associationIds = mjs.$this.$('tableField_krmybpq6').getValue().reduce((acc, cur) => {
      acc.push(...(cur.associationFormField_liya90jt || cur.associationFormField_ljnoa248).map(item => item.instanceId))
      return acc;
    }, [])
    // […new Set(arr)] 返回的是数组内set，取length异常，为1个Set对象，但循环处理数据正常。因此可通过：new Set(ids).size != ids.length
    if (new Set(associationIds).size != associationIds.length) {
      return mjs.com.toastError("请勿重复选择发票！")
    }
    // prd 费用明细中的报销金额修改金额, 不能大于发票金额
    let idx = -1;
    mjs.$this.$('tableField_krmybpq6').getValue().forEach((item, index) => {
      if (item.numberField_krn4ig4x > (item.numberField_ljau8ps7 || item.numberField_ljnoa249)) {
        idx = index;
        return
      }
    })
    if (idx >= 0) {
      return mjs.com.toastError(`第【${idx + 1}】记录, 报销金额已大于发票金额！`)
    }
    // prd 分类求和, 费用明细中的预付款金额, 不能大于预算金额
    idx = -1;
    mjs.$this.$('tableField_krmybpq6').getValue().forEach((item, index) => {
      if ((item.numberField_kxr3f3zy || item.numberField_kwd4ep08) > (item.numberField_kxr3f3zx || item.numberField_kwd4ep06)) {
        idx = index;
        return
      }
    })
    if (idx >= 0) {
      return mjs.com.toastError(`第【${idx + 1}】记录, 支付金额已大于报销金额！`)
    }
    // 兼容: 退回为监听宜搭dom事件, 先执行接口调用, 才会校验宜搭必填, 过滤无效调用 || 先匹配校验是否可调用
    const bx = (mjs.$this.$("numberField_krn54uoe") || mjs.$this.$("numberField_kroa4wk1")).getValue();
    const fk = (mjs.$this.$("numberField_krn7ufyt") || mjs.$this.$("numberField_krf2spcw")).getValue();
    if (bx != fk) {
      return mjs.com.toastError("费用明细合计与支付金额不一致！")
    }
    return true;
  }
}

