const SS = {};
const LS = {};
const CK = {};

// -------------------------------- //
//    sessionStorage - str\obj      //
// -------------------------------- //

SS.SET = function (key, data) {
  if (typeof data == "object") data = JSON.stringify(data);
  sessionStorage.setItem(key, data);
};

SS.GET = function (key) {
  let data = sessionStorage.getItem(key);
  if (/"/.test(data)) data = JSON.parse(data);
  return data;
};

SS.REMOVE = function (key) {
  sessionStorage.removeItem(key);
};

SS.CLEAR = function () {
  sessionStorage.clear();
};

// -------------------------------- //
//     localStorage - str\obj       //
// -------------------------------- //

LS.SET = function (key, data) {
  if (typeof data == "object") data = JSON.stringify(data);
  localStorage.setItem(key, data);
};

LS.GET = function (key) {
  let data = localStorage.getItem(key);
  if (/"/.test(data)) data = JSON.parse(data);
  return data;
};

LS.REMOVE = function (key) {
  localStorage.removeItem(key);
};

LS.CLEAR = function () {
  localStorage.clear();
};

// cookie 设置过多可能会导致浏览器异常
LS.SET_EXPIRE = function (key, data, expire) {
  data = {
    expire: Date.now() + expire * 1000,
    data
  }
  this.SET(key, data)
}

LS.GET_EXPIRE = function (key) {
  let data = this.GET(key);
  if (data && data.expire >= Date.now()) {
    data = data.data;
  } else {
    this.REMOVE(key)
    data = null;
  }
  return data;
}

// -------------------------------- //
//         cookies - expires        //
// -------------------------------- //

CK.SET = function (name, value, seconds) {
  if (typeof value == "object") value = JSON.stringify(value);
  let expires = "";
  if (seconds != 0) {
    //设置cookie生存时间
    const date = new Date();
    date.setTime(date.getTime() + seconds * 1000);
    expires = "; expires=" + date.toGMTString();
  }
  document.cookie = name + "=" + escape(value) + expires + "; path=/"; //转码并赋值
};

CK.GET = function (name) {
  const nameEQ = name + "=";
  const ca = document.cookie.split(";"); // 把cookie分割成组
  for (let i = 0; i < ca.length; i++) {
    let c = ca[i]; // 取得字符串
    while (c.charAt(0) == " ") {
      // 判断一下字符串有没有前导空格
      c = c.substring(1, c.length); // 有的话，从第二位开始取
    }
    if (c.indexOf(nameEQ) == 0) {
      // 如果含有需要的name
      let value = unescape(c.substring(nameEQ.length, c.length));
      if (/"/.test(value)) value = JSON.parse(value);
      return value; // 解码并截取我们要值
    }
  }
  return false;
};

CK.REMOVE = function (name) {
  this.SET(name, "", -1);
};

/* export default 导出会加一层 default 包裹:  import 导入时默认已解构
import { SS as storage } from "storage"; // import { SS } from "storage"; */
export default { SS, LS, CK };
