import * as ddAPi from "dingtalk-jsapi";
// NOTE: 未默认解构 default - 不能直接解构, 不识别. 不能有效减少包体积
const dd = ddAPi.default;

import conf from "../config/conf";

const ding = {}; // 钉钉jsapi对象
let $loading = false; // 避免多个请求闪屏

// loading 显示
ding.showLoading = function (message = "拼命加载中...") {
  if ($loading) return;
  $loading = true;
  dd.device.notification.showPreloader({
    showIcon: true,
    text: message,
  });
};

// loading 关闭
ding.hideLoading = function () {
  if (!$loading) return;
  dd.device.notification.hidePreloader();
  $loading = false;
};

// toast 提示: info 详见 https://ding-doc.dingtalk.com/doc#/dev/oo98ye/6pwsuy
ding.showMessageToast = function (info = {}) {
  if (!info.message) return;
  dd.device.notification.toast(info);
};

// 获取定位
ding.getLocation = function () {
  return new Promise((resolve, reject) => {
    dd.device.geolocation.get({
      targetAccuracy: 200,
      coordinate: 1,
      withReGeocode: true,
      useCache: true,
      onSuccess (result) {
        resolve(result);
      },
      onFail (err) {
        reject(err);
      },
    });
  });
};

// 根据chatId跳转到对应会话
ding.toConversation = function (chatId) {
  return new Promise((resolve, reject) => {
    dd.biz.chat.toConversation({
      corpId: conf.corpId,
      chatId,
      onSuccess: () => resolve(),
      onFail: (err) => reject(err),
    });
  });
};

// 图片预览: 支持缩放和左右切换
ding.previewImageScale = function (urls, index = 0) {
  if (!urls) return;
  dd.biz.previewImage({
    urls,
    current: urls[index],
  });
};

// 获取UUID
ding.getUUID = function () {
  return new Promise((resolve) => {
    dd.device.base.getUUID({
      onSuccess (data) {
        resolve(data);
      },
    });
  });
};

/** @exports 钉钉新开页面方法 */
ding.openNavigation = function (url, isReplace) {
  if (!url) return;
  // 区分环境, 兼容分享
  if (dd.env.platform === "notInDingTalk") {
    window.open(url, isReplace ? "_self" : "");
  } else {
    if (isReplace) {
      dd.biz.navigation.replace({ url });
    } else {
      dd.biz.util.openLink({ url });
    }
  }
};

import browser from "../utils/browser";

/** @exports 退出页面方法:兼容浏览器 */
ding.closeNavigationForDomEvent = function (that) {
  // 延迟: 体检 + 弹出离开确认框
  if (dd.env.platform === "notInDingTalk") {
    setTimeout(() => browser.closeCurrentTab(), 1000);
    return false;
  }
  if (dd.env.platform === "ios" || dd.env.platform === "android") {
    setTimeout(() => mjs.com.showMessage(that, "提交成功"), 350);
    setTimeout(() => dd.biz.navigation.close({}), 750);
    return true;
  }
  if (dd.env.platform === "pc") {
    // 钉钉BUG: 只在SlidePanel和Modal里起作用
    setTimeout(() => biz.navigation.quit({}), 750);
    return true;
  }
};

/** @exports 设置右侧导航栏: [{ id: "1", text: "附 件", url: "http://alading-20210318.oss-cn-shanghai.aliyuncs.com/assets/nav-link.png",},] */
ding.settingNavigationMenu = function (menus = [], isClear, callback) {
  if (isClear) {
    menus = [
      {
        id: "999",
        text: " ",
      },
    ];
  }
  dd.biz.navigation.setMenu({
    items: menus,
    onSuccess (result) {
      callback && callback(result);
    },
  });
};

// 注册钉钉js接口鉴权
ding.registerConfig = function (agentId, timeStamp, signature) {
  dd.config({
    agentId,
    corpId,
    timeStamp,
    nonceStr,
    signature,
    type: 0,
    jsApiList: ["biz.chat.toConversation"], // 注意：不要带dd。
  });
};

// 注册配置异常抛出
dd.error(function (error) {
  window.alert(JSON.stringify(error));
});

ding.dd = dd;

export default ding;
