import * as XLSXAPI from "xlsx";
import { export_json_to_excel } from "./lib/Export2Excel";
const XLSX = XLSXAPI.default;
import { format, parse } from "fecha";

///////////////////////////////////////////////// excel 导入兼容 2.0 + 3.0, 3.0 有标准导入功能 /////////////////////////////////////////////////

// 读取excel文件
function _readWorkbookFromLocalFile (file, sheetName) {
  return new Promise((resolve, reject) => {
    const reader = new FileReader();
    try {
      reader.onload = function (e) {
        const data = e.target.result;
        const workbook = XLSX.read(data, { type: "binary" });
        resolve(XLSX.utils.sheet_to_json(workbook.Sheets[sheetName]));
      };
    } catch (e) {
      reject(e.message || "文件类型不正确");
    }
    reader.readAsBinaryString(file);
  });
}

/** 宜搭明细数据处理 */
export async function _importExcelForAliworkDetail (file, sheetName, version) {
  const list = await _readWorkbookFromLocalFile(file, sheetName);
  const compId = list.shift(); // 过滤到第二行标题: 首行默认为成为数据的key
  const keys = Object.keys(compId); // 获取模板首行compId
  // 格式化数据为宜搭明细组件: fieldData: { value: "" }
  return list.map((item) => {
    keys.forEach((key) => {
      let value = item[key];
      // 时间处理: 传入内容为文本, 格式体现在表头内
      if (key.includes("dateField_")) {
        let fmt = "YYYY-MM-DD";
        const arr = compId[key].split("__");
        if (arr.length == 2) fmt = arr[1];
        value = parse(value, fmt).getTime();
      }
      // 人员处理: 格式保留和宜搭导入一致
      if (value && key.includes("employeeField_")) {
        value = value.split(",");
        value = value.reduce((acc, cur) => {
          const emp = cur.split("[");
          acc.push({ name: emp[0], value: emp[1].replace("]", "") });
          return acc;
        }, []);
        console.log("value", value);
      }
      if (version == "v2") {
        value = {
          fieldData: {
            value: value,
          },
        };
      }
      item[key] = value;
    });
    return item;
  });
}

/** @exports 宜搭明细组件和导入  */
export function createElementUpload (
  { domId = "fileReader", sheet = "Sheet1", version = "v3" } = {},
  callback,
  changeCallback
) {
  const upload = document.createElement("input");
  upload.type = "file";
  upload.name = "file";
  upload.accept = ".xlsx, .xls";
  upload.id = domId;
  upload.style = "display: none";
  upload.onchange = async function () {
    changeCallback && changeCallback();
    // 判断文件是否为空
    const file = document.getElementById("fileReader").files[0];
    if (file) {
      const details = await _importExcelForAliworkDetail(file, sheet, version);
      if (details) callback(details);
      else callback("数据格式异常");
    } else {
      callback("上传文件格式异常");
    }
  };
  document.body.append(upload);
}

///////////////////////////////////////////////// excel 2.0 导出功能 /////////////////////////////////////////////////

function formatJson (filterVal, jsonData) {
  return jsonData.map((v) => filterVal.map((j) => v[j]));
}

export function export2Excel (tHeader, filterVal, dataList, fileName) {
  const data = formatJson(filterVal, dataList);
  export_json_to_excel(tHeader, data, fileName);
}

///////////////////////////////////////////////// excel 3.0 导出功能, 标准功能在提交页面不能导入且目前还未有钩子提供, 如清除 /////////////////////////////////////////////////

//  cols承载对象,表头名称+取值字段名称, 默认取值明细所有ID
export function detailsExportExcel (
  that,
  { compId_detail, cols, fileName, sheetName, bookType } = {},
  { isClear = true } = {}
) {
  const comp = that.$(compId_detail);
  if (!cols || !cols.length) {
    cols = comp.baseRef.columns.reduce((acc, cur) => {
      acc[cur.name] = cur.label;
      return acc;
    }, {});
  }
  // TODO: 人员搜索框. 时间, 多选未兼容
  const list = comp.getValue();
  exportExcel({ cols, list, fileName, sheetName, bookType });
  // 清空明细内数据
  if (isClear) comp.setValue([]);
}

export function tabledataExportExcel (
  that,
  { compId_detail, cols, fileName, sheetName, bookType } = {}
) {
  const comp = that.$(compId_detail);
  let datakey;
  if (!cols || !cols.length) {
    cols = comp.get("columns").reduce((acc, cur) => {
      datakey = cur.dataKey;
      acc[datakey.substring(9, datakey.length)] = cur.title;
      return acc;
    }, {});
  }
  // TODO: 人员搜索框. 时间, 多选未兼容
  const data = comp.get("data").data;
  let list = new Array();
  data.forEach((item) => {
    let data_keys = Object.keys(item.formData);
    data_keys.forEach((key) => {
      if (key.includes("dateField_")) {
        item.formData[key] = format(new Date(item.formData[key]), "YYYY-MM-DD");
      }
    });
    list.push(item.formData);
  });
  exportExcel({ cols, list, fileName, sheetName, bookType });
}

import fs from "file-saver";

// 写入数据 blob 格式化
const _s2ab = (s) => {
  let buf;
  if (typeof ArrayBuffer !== "undefined") {
    buf = new ArrayBuffer(s.length);
    var view = new Uint8Array(buf);
    for (let i = 0; i != s.length; ++i) view[i] = s.charCodeAt(i) & 0xff;
    return buf;
  } else {
    buf = new Array(s.length);
    for (let i = 0; i != s.length; ++i) buf[i] = s.charCodeAt(i) & 0xff;
    return buf;
  }
};

// 前端导出 xlsx { cols: { prop: header }, list } - cols承载对象,表头名称+取值字段名称
export function exportExcel ({
  cols,
  list,
  fileName = "导出文件",
  sheetName,
  bookType = "xlsx",
} = {}) {
  // 过滤无效字段, 且导出格式为 header中文名称: 数据源值
  const props = Object.keys(cols);
  const data = list.map((item) => {
    return props.reduce((row, prop) => {
      row[cols[prop]] = item[prop];
      return row;
    }, {});
  });
  console.log("data, ", data)
  // 名称处理
  fileName = fileName;
  sheetName = sheetName || fileName;
  // 工作簿对象包含一SheetNames数组，以及一个表对象映射表名称到表对象。XLSX.utils.book_new实用函数创建一个新的工作簿对象。
  const wb = XLSX.utils.book_new();
  // 将JS对象数组转换为工作表。
  const ws = XLSX.utils.json_to_sheet(data, { header: Object.values(cols) });
  wb.SheetNames.push(sheetName);
  wb.Sheets[sheetName] = ws;
  const defaultCellStyle = {
    font: { name: "Verdana", sz: 13, color: "FF00FF88" },
    fill: { fgColor: { rgb: "FFFFAA00" } },
  };
  //设置表格的样式
  const wopts = {
    bookType: bookType,
    bookSST: false,
    type: "binary",
    cellStyles: true,
    defaultCellStyle: defaultCellStyle,
    showGridLines: false,
  };
  // 写入的样式
  const wbout = XLSX.write(wb, wopts);
  const blob = new Blob([_s2ab(wbout)], { type: "application/octet-stream" });
  fs.saveAs(blob, fileName + "." + bookType);
}
